
/* --- C ---
************************************************************************
*
*	Filename    : main.c
*	Description : main file
*	Part of     : MPEGINFO - info program for MPEG-movies
*
*	Version     : 1.0
*	Date        : Wed Oct 13 21:34:14 MDT 1993
*	Language    : C
*	For machine : INTERACTIVE Unix, Sun OS, SCO, AU/X, Solaris, Linux
*	Compile as  : see Makefile
*
*	Author      : Frank Gadegast
*	Contact     : mpeg@mpeg1.de
*	Copyright   : PHADE Software, Berlin
*
************************************************************************
*/

#include "mpeginfo.h"

extern int mpeg_info (MPEGINFO *, char *);
extern int ParseGOP (MPEGINFO *);

extern int frame;

void main(int argc, char **argv) 
{
	MPEGINFO mpeginfo;
	char cpb[20];
	int ret;

	if (!argv[1])
	{
		printf ("usage: %s filename\n", argv [0]);
		exit (-1);
	}

	printf ("\n%s reports for %s:\n\n", argv [0], argv[1]);
	if (mpeg_info (&mpeginfo, argv[1]) == PARSE_OK)
	{
		printf ("Horizontal frame size : %d\n",mpeginfo.h_size);
		printf ("Vertical frame size   : %d\n",mpeginfo.v_size);
#ifndef DOS
		printf ("Marcoblock width      : %d\n", mpeginfo.mb_width);
		printf ("Marcoblock height     : %d\n", mpeginfo.mb_height);
#endif
		printf ("\nAspect ratio          : %d\n", mpeginfo.aspect_ratio);
		printf ("Picture rate          : %d\n", mpeginfo.picture_rate);
#ifndef DOS
		printf ("Bit rate              : %d\n", mpeginfo.bit_rate);
		printf ("VBV buffer size       : %d\n", mpeginfo.vbv_buffer_size);
		printf ("Stream length         : %d\n", mpeginfo.stream_length);
#endif
		if (mpeginfo.const_param_flag) strcpy (cpb, "constrained parameter");
		else strcpy (cpb, "no constrained parameter");
		if (mpeginfo.XING) printf ("\nStream format         : XING %s\n", cpb);
		else printf ("\nStream format         : IPB %s\n", cpb);
		printf ("\n-----------------------------------\n");
		printf ("GOP Timecode          : %02d:%02d:%02d.%02d\n",
			mpeginfo.hours, mpeginfo.minutes,
			mpeginfo.seconds, mpeginfo.frame);
		while ((ret = ParseGOP (&mpeginfo)) != PARSE_ERROR)
		{
			if (ret == PARSE_OK)
				printf ("GOP Timecode          : %02d:%02d:%02d.%02d\n",
					mpeginfo.hours, mpeginfo.minutes,
					mpeginfo.seconds, mpeginfo.frame);
		}
		printf ("-----------------------------------\n");
		printf ("%d frames\n", frame+1);
	}
	else printf ("No MPEG-I-video-file.\n");
#ifndef DOS
	printf ("\n");
#endif

	exit (0);
}

